/*
	kolsko natjecanje 2011
	Zadatak: Laseri

		Opis rjeenja
	---------------------------------
	Zadatak je kolski primjer BFS problema, jedino to moramo prije se pobrinuti za lasere.

	Dakle pronaemo sve lasere koji su na istoj visini ili irini te ako
	postoji vidljiva linija izmeu njih dodamo zid, jer na lopov ne moe
	proi izmeu dva lasera. Zbog malih ogranienja mogli smo ne mariti 
	za brzinu kojom emo pronalaziti parove lasera i provjeravati vidljivost
	meu njima.

	Nakon toga dovoljno je pokrenuti BFS (breadth first search)
	nad osvjeenim poljem i dobiti vrijeme koliko je potrebno da se stigne
	od slike do ulaza/izlaza galerije.

	http://en.wikipedia.org/wiki/Breadth-first_search
*/
#include <iostream>
#include <vector>
#include <string>
#include <queue>
using namespace std;

int H, W;
vector<string> maze;
int visited[100][100] = { 0 };

pair<int, int> startPoint, endPoint;
queue<pair<int,int>> Q;

int dx[] = { 1, -1, 0, 0 };
int dy[] = { 0, 0, -1, 1 };

bool IsInsideBounds(pair<int,int> node)
{
	if (node.first < 0 || node.first >= H) return false;
	if (node.second < 0 || node.second >= W) return false;
	
	return true;
}

int BreadthFirstSearch()
{
	visited[startPoint.first][startPoint.second] = 1;

	for (Q.push(startPoint); !Q.empty(); Q.pop())
	{
		pair<int,int> node = Q.front();

		if (node == endPoint) 
			return visited[node.first][node.second];

		for (int i = 0; i < 4; ++i)
		{
			pair<int,int> newNode;

			newNode.first = node.first + dx[i];
			newNode.second = node.second + dy[i];

			if (IsInsideBounds(newNode) 
				&& maze[newNode.first][newNode.second] != 'L'
				&& maze[newNode.first][newNode.second] != '#'
				&& visited[newNode.first][newNode.second] == 0)
			{
				visited[newNode.first][newNode.second] = visited[node.first][node.second] + 1;
				Q.push(newNode);
			}
		}
	}

	return -1;
}

void DrawHorizontalLineBetween(pair<int,int> a, pair<int,int> b)
{
	int start = min(a.second, b.second);
	int end = max(a.second, b.second);

	for (int i = start+1; i < end; i++)
		maze[a.first][i] = '#';
}

void DrawVerticalLineBetween(pair<int,int> a, pair<int,int> b)
{
	int start = min(a.first, b.first);
	int end = max(a.first, b.first);

	for (int i = start+1; i < end; i++)
		maze[i][a.second] = '#';
}

bool LineOfSightExists(pair<int,int> a, pair<int,int> b, bool vertical)
{
	if(!vertical) // horizontal line
	{
		int start = min(a.second, b.second);
		int end = max(a.second, b.second);

		for (int i = start+1; i < end; i++)
			if (maze[a.first][i] == '#')
				return false;
	}
	else // vertical line
	{
		int start = min(a.first, b.first);
		int end = max(a.first, b.first);

		for (int i = start+1; i < end; i++)
			if (maze[i][a.second] == '#')
				return false;
	}

	return true;
}

void UpdateLaserWalls()
{
	vector<pair<int,int>> lasers;

	for (int i = 0; i < H; ++i)
		for (int j = 0; j < W; ++j)
			if (maze[i][j] == 'L')
				lasers.push_back(make_pair(i,j));

	for (int i = 0; i < lasers.size() - 1; ++i)
		for (int j = i+1; j < lasers.size(); ++j)
		{
			if (lasers[i].first == lasers[j].first && LineOfSightExists(lasers[i], lasers[j], false))
				DrawHorizontalLineBetween(lasers[i], lasers[j]);

			if (lasers[i].second == lasers[j].second  && LineOfSightExists(lasers[i], lasers[j], true))
				DrawVerticalLineBetween(lasers[i], lasers[j]);
		}
}

pair<int,int> FindChar(char c)
{
	for (int i = 0; i < H; ++i)
		for (int j = 0; j < W; ++j)
			if (maze[i][j] == c)
				return make_pair(i, j);

	return make_pair(-1, -1);
}

int main()
{
	cin >> H >> W;
	maze.resize(H);
	for (int i = 0; i < H; ++i )
		cin >> maze[i];
	
	startPoint = FindChar('U'); //slika
	endPoint = FindChar('S');	//ulaz

	UpdateLaserWalls();

	if( maze[startPoint.first][startPoint.second] != 'U') 
		cout << "-1" << endl;
	else 
		cout << BreadthFirstSearch() << endl;

	return 0;
}